###
### "THE BEER-WARE LICENSE":
### Alberto Ramos wrote this file. As long as you retain this 
### notice you can do whatever you want with this stuff. If we meet some 
### day, and you think this stuff is worth it, you can buy me a beer in 
### return. <alberto.ramos@cern.ch>
###
### file:    Scalar.jl
### created: Tue Oct  5 11:53:31 2021
###                               

module Scalar

using CUDA, Random, TimerOutputs
using ..Space
using ..Groups
using ..Fields
using ..MD
using ..YM
import ..YM: HMC!, randomize!, MD!, force_gauge

import Base.show

struct ScalarParm{N,T}
    kap::NTuple{N,T}
    eta::NTuple{N,T}
end
function Base.show(io::IO, sp::ScalarParm{N,T}) where {N,T}

    println(io, "Number of scalar fields: ", N)
    print(io, "Kappas: ")
    for i in 1:N
        print(io, " ", sp.kap[i])
    end
    print("\netas:   ")
    for i in 1:N
        print(io, " ", sp.eta[i])
    end
    println("\n")

end
export ScalarParm

struct ScalarWorkspace{T}
    frc1
    mom
    Phi
    function ScalarWorkspace(::Type{T}, n, lp::SpaceParm) where {T <: AbstractFloat}

        return new{T}(nscalar_field(SU2fund{T}, n, lp),
                      nscalar_field(SU2fund{T}, n, lp),
                      nscalar_field(SU2fund{T}, n, lp))
    end
end
export ScalarWorkspace

include("ScalarAction.jl")
export scalar_action

include("ScalarForce.jl")
export force_scalar

include("ScalarFields.jl")
export randomize!

include("ScalarHMC.jl")
export HMC!

end
